searchNodes=[{"ref":"Hocon.html","title":"Hocon","type":"module","doc":"This module pareses and decodes a hocon configuration string. Example iex(1)&gt; conf = ~s(animal { favorite : &quot;dog&quot; }, key : &quot;&quot;&quot;${animal.favorite} is my favorite animal&quot;&quot;&quot;) iex(2)&gt; Hocon.decode(conf) {:ok, %{&quot;animal&quot; =&gt; %{&quot;favorite&quot; =&gt; &quot;dog&quot;}, &quot;key&quot; =&gt; &quot;dog is my favorite animal&quot;}} Units format The parser returns a map, because in Elixir it is a common use case to use pattern matching on maps to extract specific values and keys. Therefore the Hocon.decode/2 function returns a map. To support interpreting a value with some family of units, you can call some conversion functions like as_bytes/1. Example iex&gt; conf = ~s(limit : &quot;512KB&quot;) iex&gt; {:ok, %{&quot;limit&quot; =&gt; limit}} = Hocon.decode(conf) iex&gt; Hocon.as_bytes(limit) 524288 It is possible to access the unit formats by a keypath, as well: Example iex&gt; conf = ~s(a { b { c { limit : &quot;512KB&quot; } } }) iex&gt; {:ok, map} = Hocon.decode(conf) iex&gt; Hocon.get_bytes(map, &quot;a.b.c.limit&quot;) 524288 iex&gt; Hocon.get_size(map, &quot;a.b.c.limit&quot;) 512000 Include HOCON supports including of other configuration files. The default implmentation uses the file systems, which seems to be the most known use case. For other use cases you can implement the Hocon.Resolver behaviour and call the decode/2 function with file_resolver: MyResolver as an option. Example The file include-1.conf exists and has the following content: { x : 10, y : ${a.x} } In the case we use the Hocon.FileResolver (which is the default as well): iex&gt; conf = ~s({ a : { include &quot;./test/data/include-1&quot; } }) iex&gt; Hocon.decode(conf, file_resolver: Hocon.FileResolver) {:ok, %{&quot;a&quot; =&gt; %{&quot;x&quot; =&gt; 10, &quot;y&quot; =&gt; 10}}} To minimize the dependencies of other packages, you need to include the HoconUrlResolver if you want to load configuration from the internet: def deps do [ {:hocon_url_resolver, &quot;~&gt; 0.1.0&quot;} ] end or just implement a resolver like: URL-Resolver with HTTPoison defmodule HoconUrlResolver do @behaviour Hocon.Resolver @spec exists?(Path.t()) :: boolean def exists?(url) do case HTTPoison.head(url) do {:ok, %HTTPoison.Response{status_code: 200}} -&gt; true {:ok, %HTTPoison.Response{status_code: 404}} -&gt; false {:error, _} -&gt; false end end def load(url) do case HTTPoison.get(url) do {:ok, %HTTPoison.Response{status_code: 200, body: body}} -&gt; {:ok, body} {:ok, %HTTPoison.Response{status_code: 404}} -&gt; {:error, &quot;not found&quot;} {:error, %HTTPoison.Error{reason: reason}} -&gt; {:error, reason} end end end"},{"ref":"Hocon.html#as_bytes/1","title":"Hocon.as_bytes/1","type":"function","doc":"Returns the size of the string by using the power of 2. Example iex&gt; Hocon.as_bytes(&quot;512kb&quot;) 524288 iex&gt; Hocon.as_bytes(&quot;125 gigabytes&quot;) 134217728000"},{"ref":"Hocon.html#as_milliseconds/1","title":"Hocon.as_milliseconds/1","type":"function","doc":"Returns the time of the string as milliseconds. Example iex&gt; Hocon.as_milliseconds(&quot;30s&quot;) 30000 iex&gt; Hocon.as_milliseconds(&quot;10us&quot;) 0.01"},{"ref":"Hocon.html#as_period/1","title":"Hocon.as_period/1","type":"function","doc":"Returns the duration of the string as Hocon.Period. Example iex&gt; Hocon.as_period(&quot;3 weeks&quot;) %Hocon.Period{days: 21, months: 0, years: 0} iex&gt; Hocon.as_period(&quot;14d&quot;) %Hocon.Period{days: 14, months: 0, years: 0}"},{"ref":"Hocon.html#as_size/1","title":"Hocon.as_size/1","type":"function","doc":"Returns the size of the string by using the power of 10. Example iex&gt; Hocon.as_size(&quot;512kb&quot;) 512000 iex&gt; Hocon.as_size(&quot;125 gigabytes&quot;) 125000000000"},{"ref":"Hocon.html#decode/2","title":"Hocon.decode/2","type":"function","doc":"Parses and decodes a hocon string and returns a map options :convert_numerically_indexed - if set to true then numerically-indexed objects are converted to arrays :strict_conversion - if set to true then numerically-indexed objects are only converted to arrays if all keys are numbers :file_resolver - set to the module, which is responsible for loading the file resources. The default is Hocon.FileResolver :url_resolver - set to the module, which is responsible for loading the url resources. The default is Hocon.FileResolver Example iex&gt; conf = ~s(animal { favorite : &quot;dog&quot; }, key : &quot;&quot;&quot;${animal.favorite} is my favorite animal&quot;&quot;&quot;) iex&gt; Hocon.decode(conf) {:ok, %{&quot;animal&quot; =&gt; %{&quot;favorite&quot; =&gt; &quot;dog&quot;}, &quot;key&quot; =&gt; &quot;dog is my favorite animal&quot;}} Runtime-Configuration with HOCON Use can use the HOCON-Parser as a Config.Provider to load configuration during boot: defmodule HOCONConfigProvider do @behaviour Config.Provider require Logger # Let&#39;s pass the path to the HOCON file as config def init(path) when is_binary(path), do: path def load(config, path) do # We need to start any app we may depend on. {:ok, _} = Application.ensure_all_started(:hocon) {:ok, _} = Application.ensure_all_started(:logger) Logger.info(&quot;Reading runtime config from \#{path}&quot;) conf = path |&gt; File.read!() |&gt; Hocon.decode!() runtime = [mailer_config(conf)] |&gt; filter_nils() Config.Reader.merge(config, runtime) end defp mailer_config(%{&quot;mailer&quot; =&gt; %{&quot;server&quot; =&gt; server, &quot;port&quot; =&gt; port}}) do {JobsKliniken.Mailer, [server: server, port: port]} end defp mailer_config(%{}) do {JobsKliniken.Mailer, nil} end defp filter_nils(keyword) do Enum.reject(keyword, fn {_key, value} -&gt; is_nil(value) end) end end"},{"ref":"Hocon.html#decode!/2","title":"Hocon.decode!/2","type":"function","doc":"Similar to decode/2 except it will unwrap the error tuple and raise in case of errors."},{"ref":"Hocon.html#get/3","title":"Hocon.get/3","type":"function","doc":"Returns a value for the keypath from a map or a successfull parse HOCON string. Example iex&gt; conf = Hocon.decode!(~s(a { b { c : &quot;10kb&quot; } })) %{&quot;a&quot; =&gt; %{&quot;b&quot; =&gt; %{&quot;c&quot; =&gt; &quot;10kb&quot;}}} iex&gt; Hocon.get(conf, &quot;a.b.c&quot;) &quot;10kb&quot; iex&gt; Hocon.get(conf, &quot;a.b.d&quot;) nil iex&gt; Hocon.get(conf, &quot;a.b.d&quot;, &quot;1kb&quot;) &quot;1kb&quot;"},{"ref":"Hocon.html#get_bytes/3","title":"Hocon.get_bytes/3","type":"function","doc":"Same a get/3 but the value is interpreted like a number by using the power of 2. Example iex&gt; conf = Hocon.decode!(~s(a { b { c : &quot;10kb&quot; } })) %{&quot;a&quot; =&gt; %{&quot;b&quot; =&gt; %{&quot;c&quot; =&gt; &quot;10kb&quot;}}} iex&gt; Hocon.get_bytes(conf, &quot;a.b.c&quot;) 10240 iex&gt; Hocon.get_bytes(conf, &quot;a.b.d&quot;) nil iex&gt; Hocon.get_bytes(conf, &quot;a.b.d&quot;, 1024) 1024"},{"ref":"Hocon.html#get_milliseconds/3","title":"Hocon.get_milliseconds/3","type":"function","doc":"Same a get/3 but the value is interpreted like a duration format in milliseconds. Example iex&gt; conf = Hocon.decode!(~s(a { b { c : &quot;30s&quot; } })) %{&quot;a&quot; =&gt; %{&quot;b&quot; =&gt; %{&quot;c&quot; =&gt; &quot;30s&quot;}}} iex&gt; Hocon.get_milliseconds(conf, &quot;a.b.c&quot;) 30000 iex&gt; Hocon.get_milliseconds(conf, &quot;a.b.d&quot;) nil iex&gt; Hocon.get_milliseconds(conf, &quot;a.b.d&quot;, 1000) 1000"},{"ref":"Hocon.html#get_period/3","title":"Hocon.get_period/3","type":"function","doc":"Same a get/3 but the value is interpreted like a duration format in Hocon.Period. Example iex&gt; conf = Hocon.decode!(~s(a { b { c : &quot;3 weeks&quot; } })) %{&quot;a&quot; =&gt; %{&quot;b&quot; =&gt; %{&quot;c&quot; =&gt; &quot;30s&quot;}}} iex&gt; Hocon.get_period(conf, &quot;a.b.c&quot;) %Hocon.Period{days: 21, months: 0, years: 0} iex&gt; Hocon.get_period(conf, &quot;a.b.d&quot;) nil iex&gt; Hocon.get_period(conf, &quot;a.b.d&quot;, 7) 7"},{"ref":"Hocon.html#get_size/3","title":"Hocon.get_size/3","type":"function","doc":"Same a get/3 but the value is interpreted like a number by using the power of 10. Example iex&gt; conf = Hocon.decode!(~s(a { b { c : &quot;10kb&quot; } })) %{&quot;a&quot; =&gt; %{&quot;b&quot; =&gt; %{&quot;c&quot; =&gt; &quot;10kb&quot;}}} iex&gt; Hocon.get_size(conf, &quot;a.b.c&quot;) 10000 iex&gt; Hocon.get_size(conf, &quot;a.b.d&quot;) nil iex&gt; Hocon.get_size(conf, &quot;a.b.d&quot;, 1000) 1000"},{"ref":"Hocon.Document.html","title":"Hocon.Document","type":"module","doc":"This module represents the necessary functions for the creation of the final configuration. The configuration is built up with put/3 function and as the final step the convert function is called, which resolves the substitutions to its current values in the current configuration object."},{"ref":"Hocon.Document.html#convert/2","title":"Hocon.Document.convert/2","type":"function","doc":"Converts the document into a map by converting numerically indexed maps into arrays resolving substitutions to it&#39;s values Example iex&gt; doc = %Document{root: %{&quot;employee&quot; =&gt; %{&quot;firstname&quot; =&gt; &quot;Michael&quot;, &quot;lastname&quot; =&gt; &quot;Maier&quot;}, &quot;fullname&quot; =&gt; &quot;${employee.firstname} ${employee.lastname}&quot;}} iex&gt; Document.convert(doc) %{ &quot;employee&quot; =&gt; %{&quot;firstname&quot; =&gt; &quot;Michael&quot;, &quot;lastname&quot; =&gt; &quot;Maier&quot;}, &quot;fullname&quot; =&gt; &quot;Michael Maier&quot; }"},{"ref":"Hocon.Document.html#get_or_fetch_env/3","title":"Hocon.Document.get_or_fetch_env/3","type":"function","doc":""},{"ref":"Hocon.Document.html#get_raw/3","title":"Hocon.Document.get_raw/3","type":"function","doc":""},{"ref":"Hocon.Document.html#get_raw/4","title":"Hocon.Document.get_raw/4","type":"function","doc":""},{"ref":"Hocon.Document.html#merge/2","title":"Hocon.Document.merge/2","type":"function","doc":""},{"ref":"Hocon.Document.html#merge/4","title":"Hocon.Document.merge/4","type":"function","doc":""},{"ref":"Hocon.Document.html#new/0","title":"Hocon.Document.new/0","type":"function","doc":""},{"ref":"Hocon.Document.html#push_key/2","title":"Hocon.Document.push_key/2","type":"function","doc":""},{"ref":"Hocon.Document.html#put/5","title":"Hocon.Document.put/5","type":"function","doc":""},{"ref":"Hocon.FileResolver.html","title":"Hocon.FileResolver","type":"module","doc":"This module is responsible for loading file resources. By implementing the behaviour Hocon.Resolver it is possible to replace this module. For example to load the resource from a database or from an url."},{"ref":"Hocon.FileResolver.html#exists?/1","title":"Hocon.FileResolver.exists?/1","type":"function","doc":"Returns true if resource exists. Example iex&gt; Hocon.FileResolver.exists?(&quot;app.conf&quot;) false iex&gt; Hocon.FileResolver.exists?(&quot;./test/data/include-1.conf&quot;) true"},{"ref":"Hocon.FileResolver.html#load/1","title":"Hocon.FileResolver.load/1","type":"function","doc":"Returns a tuple with the content of the resource Example iex&gt; Hocon.FileResolver.load(&quot;app.conf&quot;) {:error, :enoent} iex&gt; Hocon.FileResolver.load(&quot;./test/data/include-1.conf&quot;) {:ok, &quot;{ x : 10, y : ${a.x} }&quot;}"},{"ref":"Hocon.Parser.html","title":"Hocon.Parser","type":"module","doc":""},{"ref":"Hocon.Parser.html#contact_rule/2","title":"Hocon.Parser.contact_rule/2","type":"function","doc":""},{"ref":"Hocon.Parser.html#decode/2","title":"Hocon.Parser.decode/2","type":"function","doc":"Parses and decodes a hocon string and returns a map options :convert_numerically_indexed - if set to true then numerically-indexed objects are converted to arrays :strict_conversion - if set to true then numerically-indexed objects are only converted to arrays if all keys are numbers :file_resolver - a module which uses the Hocon.Resolver behaviour. If missing the Hocon.FileResolver is used as default."},{"ref":"Hocon.Parser.html#decode!/2","title":"Hocon.Parser.decode!/2","type":"function","doc":"Similar to decode/2 except it will unwrap the error tuple and raise in case of errors."},{"ref":"Hocon.Parser.html#parse/2","title":"Hocon.Parser.parse/2","type":"function","doc":""},{"ref":"Hocon.Parser.html#parse_required_close/4","title":"Hocon.Parser.parse_required_close/4","type":"function","doc":""},{"ref":"Hocon.Parser.html#parse_root/2","title":"Hocon.Parser.parse_root/2","type":"function","doc":""},{"ref":"Hocon.Parser.html#parse_value/2","title":"Hocon.Parser.parse_value/2","type":"function","doc":""},{"ref":"Hocon.Parser.html#try_concat_array/3","title":"Hocon.Parser.try_concat_array/3","type":"function","doc":""},{"ref":"Hocon.Parser.html#try_merge_object/3","title":"Hocon.Parser.try_merge_object/3","type":"function","doc":""},{"ref":"Hocon.Period.html","title":"Hocon.Period","type":"module","doc":"This structure contains the period defined by years, months and days. It is the result of the Hocon.get_period/2 and Hocon.as_period/1 function. You can use it to calculate new dates. Unfortenly Elixir Date Library does not support adding other units like months or years. It supports only days: Example iex&gt; conf = Hocon.decode!(~s(max_period = 3days)) %{&quot;max_period&quot; =&gt; &quot;3days&quot;} iex&gt; %Hocon.Period{days: days, months: _, years: _} = Hocon.get_period(conf, &quot;max_period&quot;) %Hocon.Period{days: 3, months: 0, years: 0} iex&gt; Date.add(~D[2000-02-27], days) ~D[2000-03-01] A better alternative is timex: Example iex&gt; conf = Hocon.decode!(~s(max_period = 1m)) %{&quot;max_period&quot; =&gt; &quot;1m&quot;} iex&gt; %Hocon.Period{days: _, months: months, years: _} = Hocon.get_period(conf, &quot;max_period&quot;) %Hocon.Period{days: 0, months: 1, years: 0} iex&gt; Date.add(~D[2000-02-27], days) Timex.shift(~D[2000-11-29], months: 3) ~D[2001-02-28]"},{"ref":"Hocon.Period.html#days/1","title":"Hocon.Period.days/1","type":"function","doc":"Returns a period for days."},{"ref":"Hocon.Period.html#months/1","title":"Hocon.Period.months/1","type":"function","doc":"Returns a period for months."},{"ref":"Hocon.Period.html#weeks/1","title":"Hocon.Period.weeks/1","type":"function","doc":"Returns a period for weeks. The weeks are multiplied by 7."},{"ref":"Hocon.Period.html#years/1","title":"Hocon.Period.years/1","type":"function","doc":"Returns a period for years."},{"ref":"Hocon.Resolver.html","title":"Hocon.Resolver","type":"behaviour","doc":"This module is responsible for loading file configuration. The configuration is specified by a path (filepath, url). By implementing this behaviour you can pass the module by the :resolver keyword as an option. Example iex&gt; conf = ~s({ a : { include required(&quot;./test/data/include-3&quot;) } } iex&gt; Hocon.decode(conf), resolver: Hocon.FileResolver) {:ok, %{&quot;a&quot; =&gt; %{&quot;x&quot; =&gt; 10, &quot;y&quot; =&gt; 10}}}"},{"ref":"Hocon.Resolver.html#c:exists?/1","title":"Hocon.Resolver.exists?/1","type":"callback","doc":"Returns true if the given path exists."},{"ref":"Hocon.Resolver.html#c:load/1","title":"Hocon.Resolver.load/1","type":"callback","doc":"Returns {:ok, binary}, where binary is a binary data object that contains the contents of path, or {:error, reason} if an error occurs."},{"ref":"Hocon.Tokenizer.html","title":"Hocon.Tokenizer","type":"module","doc":""},{"ref":"Hocon.Tokenizer.html#decode/1","title":"Hocon.Tokenizer.decode/1","type":"function","doc":""},{"ref":"Hocon.Tokenizer.html#multi_line_string/5","title":"Hocon.Tokenizer.multi_line_string/5","type":"function","doc":""},{"ref":"Hocon.Tokenizer.html#multi_line_string_end/5","title":"Hocon.Tokenizer.multi_line_string_end/5","type":"function","doc":""},{"ref":"Hocon.Tokenizer.html#multi_line_string_start/5","title":"Hocon.Tokenizer.multi_line_string_start/5","type":"function","doc":""},{"ref":"Hocon.Tokenizer.html#string/5","title":"Hocon.Tokenizer.string/5","type":"function","doc":""},{"ref":"Hocon.Tokenizer.html#string/6","title":"Hocon.Tokenizer.string/6","type":"function","doc":""},{"ref":"Hocon.Tokenizer.html#substitutions/5","title":"Hocon.Tokenizer.substitutions/5","type":"function","doc":""},{"ref":"Hocon.Tokenizer.html#tokenize/4","title":"Hocon.Tokenizer.tokenize/4","type":"function","doc":""},{"ref":"Hocon.Tokenizer.html#unquoted_string/5","title":"Hocon.Tokenizer.unquoted_string/5","type":"function","doc":""},{"ref":"Hocon.Tokens.html","title":"Hocon.Tokens","type":"module","doc":"This module is responsible for pushing tokens to the list of tokens. It handles the cases of whitespaces and new lines. Whitespaces are important in cases of unquoted strings und new lines are important for merging objects and arrays. whitespace are almost ignored only allowed in context of unquoted strings and simple values see new lines are used to see See also the tokens_test.exs for more information."},{"ref":"Hocon.Tokens.html#new/0","title":"Hocon.Tokens.new/0","type":"function","doc":"Create a new Tokens struct. Example iex(1)&gt; Hocon.Tokens.new() %Hocon.Tokens{acc: [], ws: false}"},{"ref":"Hocon.Tokens.html#push/2","title":"Hocon.Tokens.push/2","type":"function","doc":"Push a token to the Tokens struct. Example tokens = Tokens.new() seq = tokens |&gt; Tokens.push(1) |&gt; Tokens.push(:ws) |&gt; Tokens.push({:unquoted_string, &quot;2&quot;}) |&gt; Tokens.push(:ws) |&gt; Tokens.push(3) |&gt; Tokens.push(:ws) |&gt; Tokens.push({:unquoted_string, &quot;4&quot;}) assert [{:unquoted_string, &quot;4&quot;}, :ws, 3, :ws, {:unquoted_string, &quot;2&quot;}, :ws, 1] == seq.acc"},{"ref":"readme.html","title":"hocon","type":"extras","doc":"hocon Parse HOCON configuration files in Elixir following the HOCON specifications."},{"ref":"readme.html#basic-usage","title":"hocon - Basic usage","type":"extras","doc":"Assume the file my-configuration.conf exists and has the following content: { home : /path/to/home timeout : 300 logger { level = &quot;DEBUG&quot; } } Then you can read and parse the HOCON-Configuration file: {:ok, body} = File.read(&quot;my-configuration.conf&quot;) result = Hocon.decode(body) IO.puts inspect result {:ok, %{&quot;home&quot; =&gt; &quot;path/to/home&quot;, &quot;logger&quot; =&gt; %{&quot;level&quot; =&gt; &quot;DEBUG&quot;}, &quot;timeout&quot; =&gt; 300}} The HOCON configuration is very powerfull and has a lot of nice features { // you can use comments # you can concat arrays like this dirs += ${PWD} dirs += /working-folder # you can concat strings like this path : ${PWD} path : ${path}&quot;/working-folder&quot; # Here are several ways to define `a` to the same array value: // one array a : [ 1, 2, 3, 4 ] // two arrays that are concatenated a : [ 1, 2 ] [ 3, 4 ] // with self-referential substitutions a : [ 1, 2 ] a : ${a} [ 3, 4 ] # some nested objects: foo { bar { baz : 42 } } # you can build values with substitutions foo : { a : { c : 1 } } foo : ${foo.a} foo : { a : 2 } } After parsing you get this map as result (where PWD=/Users/micha/projects/elixir/hocon): %{ &quot;dirs&quot; =&gt; [&quot;/Users/micha/projects/elixir/hocon&quot;, &quot;working-folder&quot;], &quot;path&quot; =&gt; &quot;/Users/micha/projects/elixir/hocon/working-folder&quot;}, &quot;a&quot; =&gt; [1, 2, 3, 4], &quot;foo&quot; =&gt; %{&quot;a&quot; =&gt; 2, &quot;bar&quot; =&gt; %{&quot;baz&quot; =&gt; 42}, &quot;c&quot; =&gt; 1} }"},{"ref":"readme.html#spec-coverage","title":"hocon - Spec Coverage","type":"extras","doc":"https://github.com/lightbend/config/blob/master/HOCON.md [x] parsing JSON [x] comments [x] omit root braces [x] key-value separator [x] commas are optional if newline is present [x] whitespace [x] duplicate keys and object merging [x] unquoted strings [x] multi-line strings [x] value concatenation [x] object concatenation [x] array concatenation [x] path expressions [x] path as keys [x] substitutions [x] includes [x] conversion of numerically-indexed objects to arrays [x] allow URL for included files [x] duration unit format [x] period unit format [x] size unit format"}]